==> rhythmbox-0.9.1.tar.gz: rhythmbox-0.9.1/shell/rb-source-header.c <==
/* 
 *  arch-tag: Implementation of search entry/browse toggle container
 *
 *  Copyright (C) 2003 Jorn Baayen <jorn@nl.linux.org>
 *  Copyright (C) 2003,2004 Colin Walters <walters@redhat.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <gtk/gtk.h>
#include <config.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <libgnome/gnome-i18n.h>

#include "disclosure-widget.h"
#include "rb-source-header.h"
#include "rb-stock-icons.h"
#include "rb-preferences.h"
#include "rb-search-entry.h"
#include "rb-debug.h"
#include "rb-remote.h"
#include "rb-entry-view.h"
#include "eel-gconf-extensions.h"

static void rb_source_header_class_init (RBSourceHeaderClass *klass);
static void rb_source_header_init (RBSourceHeader *shell_player);
static void rb_source_header_finalize (GObject *object);
static void rb_source_header_set_property (GObject *object,
					  guint prop_id,
					  const GValue *value,
					  GParamSpec *pspec);
static void rb_source_header_get_property (GObject *object,
					  guint prop_id,
					  GValue *value,
					  GParamSpec *pspec);
static void rb_source_header_filter_changed_cb (RBSource *source,
						RBSourceHeader *header);
static void rb_source_header_search_cb (RBSearchEntry *search,
					const char *text,
					RBSourceHeader *header);
static void rb_source_header_disclosure_toggled_cb (GObject *object,
						    GParamSpec *param_spec,
						    gpointer data);
static void rb_source_header_search_activate_cb (RBSearchEntry *search,
						 RBSourceHeader *header);
static void rb_source_header_gconf_disclosure_changed_cb (GConfClient *client,
							  guint cnxn_id,
							  GConfEntry *entry,
							  RBSourceHeader *header);
static void rb_source_header_view_browser_changed_cb (GtkAction *action,
						      RBSourceHeader *header);

struct RBSourceHeaderPrivate
{
	RBSource *selected_source;

	GtkActionGroup *actiongroup;

	GtkTooltips *tooltips;

	GtkWidget *search;
	GtkWidget *disclosure;

	guint browser_notify_id;
	guint search_notify_id;
	gboolean have_search;
	const char *browser_key;
};

enum
{
	PROP_0,
	PROP_ACTION_GROUP,
	PROP_SOURCE,
};

static GtkToggleActionEntry rb_source_header_toggle_entries [] =
{
	{ "ViewBrowser", NULL, N_("_Browser"), "<control>B",
	  N_("Change the visibility of the browser"),
	  G_CALLBACK (rb_source_header_view_browser_changed_cb) }
};
static guint rb_source_header_n_toggle_entries = G_N_ELEMENTS (rb_source_header_toggle_entries);


static GObjectClass *parent_class = NULL;

GType
rb_source_header_get_type (void)
{
	static GType rb_source_header_type = 0;

	if (rb_source_header_type == 0)
	{
		static const GTypeInfo our_info =
		{
			sizeof (RBSourceHeaderClass),
			NULL,
			NULL,
			(GClassInitFunc) rb_source_header_class_init,
			NULL,
			NULL,
			sizeof (RBSourceHeader),
			0,
			(GInstanceInitFunc) rb_source_header_init
		};

		rb_source_header_type = g_type_register_static (GTK_TYPE_TABLE,
							       "RBSourceHeader",
							       &our_info, 0);
	}

	return rb_source_header_type;
}

static void
rb_source_header_class_init (RBSourceHeaderClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);

	parent_class = g_type_class_peek_parent (klass);

	object_class->finalize = rb_source_header_finalize;

	object_class->set_property = rb_source_header_set_property;
	object_class->get_property = rb_source_header_get_property;

	g_object_class_install_property (object_class,
					 PROP_SOURCE,
					 g_param_spec_object ("source",
							      "RBSource",
							      "RBSource object",
							      RB_TYPE_SOURCE,
							      G_PARAM_READWRITE));
	g_object_class_install_property (object_class,
					 PROP_ACTION_GROUP,
					 g_param_spec_object ("action-group",
							      "GtkActionGroup",
							      "GtkActionGroup object",
							      GTK_TYPE_ACTION_GROUP,
							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
}

static void
rb_source_header_init (RBSourceHeader *header)
{
	GtkWidget *align;
	GtkEventBox *ebox;
	header->priv = g_new0 (RBSourceHeaderPrivate, 1);

	header->priv->tooltips = gtk_tooltips_new ();
	gtk_tooltips_enable (header->priv->tooltips);

	gtk_table_set_homogeneous (GTK_TABLE (header), TRUE);
	gtk_table_set_col_spacings (GTK_TABLE (header), 5);
	gtk_table_resize (GTK_TABLE (header), 1, 3);

	ebox = GTK_EVENT_BOX (gtk_event_box_new ());
	header->priv->search = GTK_WIDGET (rb_search_entry_new ());
	gtk_tooltips_set_tip (GTK_TOOLTIPS (header->priv->tooltips), 
			      GTK_WIDGET (ebox), 
			      _("Filter music display by genre, artist, album, or title"),
			      NULL);
	gtk_container_add (GTK_CONTAINER (ebox), GTK_WIDGET (header->priv->search));

	g_signal_connect_object (G_OBJECT (header->priv->search), "search",
				 G_CALLBACK (rb_source_header_search_cb), header, 0);
	g_signal_connect_object (G_OBJECT (header->priv->search), "activate",
				 G_CALLBACK (rb_source_header_search_activate_cb), header, 0);

	header->priv->disclosure = cddb_disclosure_new (_("Show _Browser"),
							_("Hide _Browser"));
	gtk_widget_set_sensitive (header->priv->disclosure, FALSE);
	g_signal_connect_object (G_OBJECT (header->priv->disclosure), "notify::expanded",
				 G_CALLBACK (rb_source_header_disclosure_toggled_cb), header, 0);

	gtk_table_attach_defaults (GTK_TABLE (header),
			           header->priv->disclosure, 0, 2, 0, 1);

	align = gtk_alignment_new (1.0, 0.5, 1.0, 1.0);
	gtk_container_add (GTK_CONTAINER (align), GTK_WIDGET (ebox));
	gtk_table_attach (GTK_TABLE (header),
			  align, 2, 3, 0, 1, 
			  GTK_EXPAND | GTK_FILL,
			  GTK_EXPAND | GTK_FILL,
			  5, 0);
}

static void
rb_source_header_finalize (GObject *object)
{
	RBSourceHeader *header;

	g_return_if_fail (object != NULL);
	g_return_if_fail (RB_IS_SOURCE_HEADER (object));

	header = RB_SOURCE_HEADER (object);

	g_return_if_fail (header->priv != NULL);

	g_free (header->priv);

	G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
rb_source_header_set_property (GObject *object,
			      guint prop_id,
			      const GValue *value,
			      GParamSpec *pspec)
{
	RBSourceHeader *header = RB_SOURCE_HEADER (object);

	switch (prop_id)
	{
	case PROP_SOURCE:
		if (header->priv->selected_source != NULL)
		{
			if (header->priv->browser_key)
				eel_gconf_notification_remove (header->priv->browser_notify_id);
			g_signal_handlers_disconnect_by_func (G_OBJECT (header->priv->selected_source),
							      G_CALLBACK (rb_source_header_filter_changed_cb),
							      header);
		}

		header->priv->selected_source = g_value_get_object (value);
		rb_debug ("selected source %p", g_value_get_object (value));

		if (header->priv->selected_source != NULL)
		{
			header->priv->browser_key = rb_source_get_browser_key (header->priv->selected_source);
			if (header->priv->browser_key)
				header->priv->browser_notify_id
					= eel_gconf_notification_add (header->priv->browser_key,
								      (GConfClientNotifyFunc) rb_source_header_gconf_disclosure_changed_cb,
								      header);
			rb_search_entry_set_text (RB_SEARCH_ENTRY (header->priv->search), "");
			g_signal_connect_object (G_OBJECT (header->priv->selected_source),
						 "filter_changed",
						 G_CALLBACK (rb_source_header_filter_changed_cb),
						 header, 0);
			gtk_widget_set_sensitive (GTK_WIDGET (header->priv->search),
						  rb_source_can_search (header->priv->selected_source));
			header->priv->have_search = rb_source_can_search (header->priv->selected_source);
			if (!header->priv->browser_key && !header->priv->have_search)
				gtk_widget_hide (GTK_WIDGET (header));
			else
				gtk_widget_show (GTK_WIDGET (header));
			if (header->priv->have_search)
				rb_source_header_search_cb (RB_SEARCH_ENTRY (header->priv->search),
							    "",
							    header);
		}
		rb_source_header_sync_control_state (header);
		
		break;
	case PROP_ACTION_GROUP:
		header->priv->actiongroup = g_value_get_object (value);
		gtk_action_group_add_toggle_actions (header->priv->actiongroup,
						     rb_source_header_toggle_entries,
						     rb_source_header_n_toggle_entries,
						     header);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
		break;
	}
}

static void 
rb_source_header_get_property (GObject *object,
			      guint prop_id,
			      GValue *value,
			      GParamSpec *pspec)
{
	RBSourceHeader *header = RB_SOURCE_HEADER (object);

	switch (prop_id)
	{
	case PROP_SOURCE:
		g_value_set_object (value, header->priv->selected_source);
		break;
	case PROP_ACTION_GROUP:
		g_value_set_object (value, header->priv->actiongroup);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
		break;
	}
}

void
rb_source_header_set_source (RBSourceHeader *header,
			    RBSource *source)
{
	g_return_if_fail (RB_IS_SOURCE_HEADER (header));
	g_return_if_fail (RB_IS_SOURCE (source));

	g_object_set (G_OBJECT (header),
		      "source", source,
		      NULL);
}

RBSourceHeader *
rb_source_header_new (GtkActionGroup *actiongroup)
{
	RBSourceHeader *header = g_object_new (RB_TYPE_SOURCE_HEADER,
					       "action-group", actiongroup,
					       NULL);

	g_return_val_if_fail (header->priv != NULL, NULL);

	return header;
}

static void
rb_source_header_filter_changed_cb (RBSource *source,
				   RBSourceHeader *header)
{
	rb_debug  ("filter changed for %p", source);
	/* To add this line back in, you need to add a search_changed signal,
	 * make RBShufflePlayOrder at least listen to it, and change the
	 * filter_changed notifies in all of the search functions to
	 * search_change notifies.
	 */
	/* rb_search_entry_clear (RB_SEARCH_ENTRY (header->priv->search)); */
}

static void
rb_source_header_search_cb (RBSearchEntry *search,
			    const char *text,
			    RBSourceHeader *header)
{
	rb_debug  ("searching for \"%s\"", text);
	rb_source_search (header->priv->selected_source, text);
	rb_source_header_sync_control_state (header);
}

void
rb_source_header_clear_search (RBSourceHeader *header)
{
	rb_debug ("clearing search");

	if (!rb_search_entry_searching (RB_SEARCH_ENTRY (header->priv->search)))
	    return;
	
	if (header->priv->selected_source)
		rb_source_search (header->priv->selected_source, NULL);
	rb_search_entry_clear (RB_SEARCH_ENTRY (header->priv->search));
	rb_source_header_sync_control_state (header);
}

static void
rb_source_header_disclosure_toggled_cb (GObject *object,
					GParamSpec *param_spec,
					gpointer data)
{
	RBSourceHeader *header = RB_SOURCE_HEADER (data);
	GtkExpander *expander = GTK_EXPANDER (object);
	gboolean disclosed = gtk_expander_get_expanded (expander);

	rb_debug ("disclosed: %s", disclosed ? "TRUE" : "FALSE");
	
	if (header->priv->selected_source != NULL)
		eel_gconf_set_boolean (header->priv->browser_key, disclosed);
}

static void
rb_source_header_gconf_disclosure_changed_cb (GConfClient *client,
					      guint cnxn_id,
					      GConfEntry *entry,
					      RBSourceHeader *header)
{
	rb_debug ("gconf disclosure changed");

	g_return_if_fail (header->priv->browser_key != NULL);

	rb_source_header_sync_control_state (header);
}

static void
rb_source_header_view_browser_changed_cb (GtkAction *action,
					  RBSourceHeader *header)
{
	rb_debug ("got view browser toggle");
	eel_gconf_set_boolean (header->priv->browser_key,
			       gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
}

void
rb_source_header_sync_control_state (RBSourceHeader *header)
{
	GtkAction *viewbrowser_action;
	GtkAction *viewstatusbar_action;
	GtkAction *viewall_action;
	gboolean have_browser = header->priv->selected_source != NULL
		&& header->priv->browser_key != NULL;
	gboolean not_small = !eel_gconf_get_boolean (CONF_UI_SMALL_DISPLAY);

	gtk_widget_set_sensitive (header->priv->disclosure,
				  have_browser);
	viewbrowser_action = gtk_action_group_get_action (header->priv->actiongroup,
							  "ViewBrowser");
	g_object_set (G_OBJECT (viewbrowser_action), "sensitive",
		      have_browser && not_small, NULL);
	viewstatusbar_action = gtk_action_group_get_action (header->priv->actiongroup,
							    "ViewStatusbar");
	g_object_set (G_OBJECT (viewstatusbar_action), "sensitive",
		      not_small && header->priv->have_search, NULL);
	viewall_action = gtk_action_group_get_action (header->priv->actiongroup,
						      "ViewAll");
	g_object_set (G_OBJECT (viewall_action), "sensitive",
		      have_browser && not_small, NULL);
	if (have_browser) {
		gboolean shown = eel_gconf_get_boolean (header->priv->browser_key);
		gtk_expander_set_expanded    (GTK_EXPANDER (header->priv->disclosure),
					      shown);
		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (viewbrowser_action),
					      shown);
	}
}

static void
rb_source_header_search_activate_cb (RBSearchEntry *search,
				     RBSourceHeader *header)
{
	RBEntryView *entry_view;

	entry_view = rb_source_get_entry_view (header->priv->selected_source);
	gtk_widget_grab_focus (GTK_WIDGET (entry_view));
}

==> rhythmbox-0.9.2.tar.gz: rhythmbox-0.9.2/shell/rb-source-header.c <==
/* 
 *  arch-tag: Implementation of search entry/browse toggle container
 *
 *  Copyright (C) 2003 Jorn Baayen <jorn@nl.linux.org>
 *  Copyright (C) 2003,2004 Colin Walters <walters@redhat.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <gtk/gtk.h>
#include <config.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <libgnome/gnome-i18n.h>

#include "disclosure-widget.h"
#include "rb-source-header.h"
#include "rb-stock-icons.h"
#include "rb-preferences.h"
#include "rb-search-entry.h"
#include "rb-debug.h"
#include "rb-remote.h"
#include "rb-entry-view.h"
#include "eel-gconf-extensions.h"

static void rb_source_header_class_init (RBSourceHeaderClass *klass);
static void rb_source_header_init (RBSourceHeader *shell_player);
static void rb_source_header_finalize (GObject *object);
static void rb_source_header_set_property (GObject *object,
					  guint prop_id,
					  const GValue *value,
					  GParamSpec *pspec);
static void rb_source_header_get_property (GObject *object,
					  guint prop_id,
					  GValue *value,
					  GParamSpec *pspec);
static void rb_source_header_filter_changed_cb (RBSource *source,
						RBSourceHeader *header);
static void rb_source_header_search_cb (RBSearchEntry *search,
					const char *text,
					RBSourceHeader *header);
static void rb_source_header_disclosure_toggled_cb (GObject *object,
						    GParamSpec *param_spec,
						    gpointer data);
static void rb_source_header_search_activate_cb (RBSearchEntry *search,
						 RBSourceHeader *header);
static void rb_source_header_gconf_disclosure_changed_cb (GConfClient *client,
							  guint cnxn_id,
							  GConfEntry *entry,
							  RBSourceHeader *header);
static void rb_source_header_view_browser_changed_cb (GtkAction *action,
						      RBSourceHeader *header);
static void rb_source_header_source_weak_destroy_cb (RBSourceHeader *header, RBSource *source);

struct RBSourceHeaderPrivate
{
	RBSource *selected_source;

	GtkActionGroup *actiongroup;

	GtkTooltips *tooltips;

	GtkWidget *search;
	GtkWidget *disclosure;

	guint browser_notify_id;
	guint search_notify_id;
	gboolean have_search;
	const char *browser_key;

	GHashTable *source_search_text;
};

enum
{
	PROP_0,
	PROP_ACTION_GROUP,
	PROP_SOURCE,
};

static GtkToggleActionEntry rb_source_header_toggle_entries [] =
{
	{ "ViewBrowser", NULL, N_("_Browser"), "<control>B",
	  N_("Change the visibility of the browser"),
	  G_CALLBACK (rb_source_header_view_browser_changed_cb) }
};
static guint rb_source_header_n_toggle_entries = G_N_ELEMENTS (rb_source_header_toggle_entries);


G_DEFINE_TYPE (RBSourceHeader, rb_source_header, GTK_TYPE_TABLE)

static void
rb_source_header_class_init (RBSourceHeaderClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);

	object_class->finalize = rb_source_header_finalize;

	object_class->set_property = rb_source_header_set_property;
	object_class->get_property = rb_source_header_get_property;

	g_object_class_install_property (object_class,
					 PROP_SOURCE,
					 g_param_spec_object ("source",
							      "RBSource",
							      "RBSource object",
							      RB_TYPE_SOURCE,
							      G_PARAM_READWRITE));
	g_object_class_install_property (object_class,
					 PROP_ACTION_GROUP,
					 g_param_spec_object ("action-group",
							      "GtkActionGroup",
							      "GtkActionGroup object",
							      GTK_TYPE_ACTION_GROUP,
							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
}

static void
rb_source_header_init (RBSourceHeader *header)
{
	GtkWidget *align;
	GtkEventBox *ebox;
	header->priv = g_new0 (RBSourceHeaderPrivate, 1);

	header->priv->tooltips = gtk_tooltips_new ();
	gtk_tooltips_enable (header->priv->tooltips);

	gtk_table_set_homogeneous (GTK_TABLE (header), TRUE);
	gtk_table_set_col_spacings (GTK_TABLE (header), 5);
	gtk_table_resize (GTK_TABLE (header), 1, 3);

	ebox = GTK_EVENT_BOX (gtk_event_box_new ());
	header->priv->search = GTK_WIDGET (rb_search_entry_new ());
	gtk_tooltips_set_tip (GTK_TOOLTIPS (header->priv->tooltips), 
			      GTK_WIDGET (ebox), 
			      _("Filter music display by genre, artist, album, or title"),
			      NULL);
	gtk_container_add (GTK_CONTAINER (ebox), GTK_WIDGET (header->priv->search));

	g_signal_connect_object (G_OBJECT (header->priv->search), "search",
				 G_CALLBACK (rb_source_header_search_cb), header, 0);
	g_signal_connect_object (G_OBJECT (header->priv->search), "activate",
				 G_CALLBACK (rb_source_header_search_activate_cb), header, 0);

	header->priv->disclosure = cddb_disclosure_new (_("Show _Browser"),
							_("Hide _Browser"));
	gtk_widget_set_sensitive (header->priv->disclosure, FALSE);
	g_signal_connect_object (G_OBJECT (header->priv->disclosure), "notify::expanded",
				 G_CALLBACK (rb_source_header_disclosure_toggled_cb), header, 0);

	gtk_table_attach_defaults (GTK_TABLE (header),
			           header->priv->disclosure, 0, 2, 0, 1);

	align = gtk_alignment_new (1.0, 0.5, 1.0, 1.0);
	gtk_container_add (GTK_CONTAINER (align), GTK_WIDGET (ebox));
	gtk_table_attach (GTK_TABLE (header),
			  align, 2, 3, 0, 1, 
			  GTK_EXPAND | GTK_FILL,
			  GTK_EXPAND | GTK_FILL,
			  5, 0);

	header->priv->source_search_text = g_hash_table_new_full (g_direct_hash, g_direct_equal,
								  NULL, (GDestroyNotify)g_free);
}

static void
rb_source_header_source_weak_unref (RBSource *source, char *text, RBSourceHeader *header)
{
	g_object_weak_unref (G_OBJECT (source),
			     (GWeakNotify)rb_source_header_source_weak_destroy_cb,
			     header);
}

static void
rb_source_header_finalize (GObject *object)
{
	RBSourceHeader *header;

	g_return_if_fail (object != NULL);
	g_return_if_fail (RB_IS_SOURCE_HEADER (object));

	header = RB_SOURCE_HEADER (object);

	g_return_if_fail (header->priv != NULL);

	g_hash_table_foreach (header->priv->source_search_text,
			      (GHFunc) rb_source_header_source_weak_unref,
			      header);
	g_hash_table_destroy (header->priv->source_search_text);

	g_free (header->priv);

	G_OBJECT_CLASS (rb_source_header_parent_class)->finalize (object);
}

static void
rb_source_header_set_property (GObject *object,
			      guint prop_id,
			      const GValue *value,
			      GParamSpec *pspec)
{
	RBSourceHeader *header = RB_SOURCE_HEADER (object);

	switch (prop_id)
	{
	case PROP_SOURCE:
		if (header->priv->selected_source != NULL)
		{
			if (header->priv->browser_key)
				eel_gconf_notification_remove (header->priv->browser_notify_id);
			g_signal_handlers_disconnect_by_func (G_OBJECT (header->priv->selected_source),
							      G_CALLBACK (rb_source_header_filter_changed_cb),
							      header);
		}

		header->priv->selected_source = g_value_get_object (value);
		rb_debug ("selected source %p", g_value_get_object (value));

		if (header->priv->selected_source != NULL)
		{
			const char *text = g_hash_table_lookup (header->priv->source_search_text,
								header->priv->selected_source);
			
			header->priv->browser_key = rb_source_get_browser_key (header->priv->selected_source);
			if (header->priv->browser_key)
				header->priv->browser_notify_id
					= eel_gconf_notification_add (header->priv->browser_key,
								      (GConfClientNotifyFunc) rb_source_header_gconf_disclosure_changed_cb,
								      header);
			rb_search_entry_set_text (RB_SEARCH_ENTRY (header->priv->search), text);
			g_signal_connect_object (G_OBJECT (header->priv->selected_source),
						 "filter_changed",
						 G_CALLBACK (rb_source_header_filter_changed_cb),
						 header, 0);
			gtk_widget_set_sensitive (GTK_WIDGET (header->priv->search),
						  rb_source_can_search (header->priv->selected_source));
			header->priv->have_search = rb_source_can_search (header->priv->selected_source);
			if (!header->priv->browser_key && !header->priv->have_search)
				gtk_widget_hide (GTK_WIDGET (header));
			else
				gtk_widget_show (GTK_WIDGET (header));
		}
		rb_source_header_sync_control_state (header);
		
		break;
	case PROP_ACTION_GROUP:
		header->priv->actiongroup = g_value_get_object (value);
		gtk_action_group_add_toggle_actions (header->priv->actiongroup,
						     rb_source_header_toggle_entries,
						     rb_source_header_n_toggle_entries,
						     header);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
		break;
	}
}

static void 
rb_source_header_get_property (GObject *object,
			      guint prop_id,
			      GValue *value,
			      GParamSpec *pspec)
{
	RBSourceHeader *header = RB_SOURCE_HEADER (object);

	switch (prop_id)
	{
	case PROP_SOURCE:
		g_value_set_object (value, header->priv->selected_source);
		break;
	case PROP_ACTION_GROUP:
		g_value_set_object (value, header->priv->actiongroup);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
		break;
	}
}

void
rb_source_header_set_source (RBSourceHeader *header,
			    RBSource *source)
{
	g_return_if_fail (RB_IS_SOURCE_HEADER (header));
	g_return_if_fail (RB_IS_SOURCE (source));

	g_object_set (G_OBJECT (header),
		      "source", source,
		      NULL);
}

RBSourceHeader *
rb_source_header_new (GtkActionGroup *actiongroup)
{
	RBSourceHeader *header = g_object_new (RB_TYPE_SOURCE_HEADER,
					       "action-group", actiongroup,
					       NULL);

	g_return_val_if_fail (header->priv != NULL, NULL);

	return header;
}

static void
rb_source_header_filter_changed_cb (RBSource *source,
				   RBSourceHeader *header)
{
	rb_debug  ("filter changed for %p", source);
	/* To add this line back in, you need to add a search_changed signal,
	 * make RBShufflePlayOrder at least listen to it, and change the
	 * filter_changed notifies in all of the search functions to
	 * search_change notifies.
	 */
	/* rb_search_entry_clear (RB_SEARCH_ENTRY (header->priv->search)); */
}

static void
rb_source_header_source_weak_destroy_cb (RBSourceHeader *header, RBSource *source)
{
	g_hash_table_remove (header->priv->source_search_text, source);
}

static void
rb_source_header_search_cb (RBSearchEntry *search,
			    const char *text,
			    RBSourceHeader *header)
{
	rb_debug  ("searching for \"%s\"", text);

	/* if we haven't seen the source before, monitor it for deletion */
	if (g_hash_table_lookup (header->priv->source_search_text, header->priv->selected_source) == NULL) {
		g_object_weak_ref (G_OBJECT (header->priv->selected_source),
				   (GWeakNotify)rb_source_header_source_weak_destroy_cb,
				   header);
	}

	g_hash_table_insert (header->priv->source_search_text,
			     header->priv->selected_source,
			     g_strdup (text));
	rb_source_search (header->priv->selected_source, text);
	rb_source_header_sync_control_state (header);
}

void
rb_source_header_clear_search (RBSourceHeader *header)
{
	rb_debug ("clearing search");

	if (!rb_search_entry_searching (RB_SEARCH_ENTRY (header->priv->search)))
	    return;
	
	if (header->priv->selected_source)
		rb_source_search (header->priv->selected_source, NULL);
	rb_search_entry_clear (RB_SEARCH_ENTRY (header->priv->search));
	rb_source_header_sync_control_state (header);
}

static void
rb_source_header_disclosure_toggled_cb (GObject *object,
					GParamSpec *param_spec,
					gpointer data)
{
	RBSourceHeader *header = RB_SOURCE_HEADER (data);
	GtkExpander *expander = GTK_EXPANDER (object);
	gboolean disclosed = gtk_expander_get_expanded (expander);

	rb_debug ("disclosed: %s", disclosed ? "TRUE" : "FALSE");
	
	if (header->priv->selected_source != NULL)
		eel_gconf_set_boolean (header->priv->browser_key, disclosed);
}

static void
rb_source_header_gconf_disclosure_changed_cb (GConfClient *client,
					      guint cnxn_id,
					      GConfEntry *entry,
					      RBSourceHeader *header)
{
	rb_debug ("gconf disclosure changed");

	g_return_if_fail (header->priv->browser_key != NULL);

	rb_source_header_sync_control_state (header);
}

static void
rb_source_header_view_browser_changed_cb (GtkAction *action,
					  RBSourceHeader *header)
{
	rb_debug ("got view browser toggle");
	eel_gconf_set_boolean (header->priv->browser_key,
			       gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
}

void
rb_source_header_sync_control_state (RBSourceHeader *header)
{
	GtkAction *viewbrowser_action;
	GtkAction *viewstatusbar_action;
	GtkAction *viewall_action;
	gboolean have_browser = header->priv->selected_source != NULL
		&& header->priv->browser_key != NULL;
	gboolean not_small = !eel_gconf_get_boolean (CONF_UI_SMALL_DISPLAY);

	gtk_widget_set_sensitive (header->priv->disclosure,
				  have_browser);
	viewbrowser_action = gtk_action_group_get_action (header->priv->actiongroup,
							  "ViewBrowser");
	g_object_set (G_OBJECT (viewbrowser_action), "sensitive",
		      have_browser && not_small, NULL);
	viewstatusbar_action = gtk_action_group_get_action (header->priv->actiongroup,
							    "ViewStatusbar");
	g_object_set (G_OBJECT (viewstatusbar_action), "sensitive",
		      not_small, NULL);
	viewall_action = gtk_action_group_get_action (header->priv->actiongroup,
						      "ViewAll");
	g_object_set (G_OBJECT (viewall_action), "sensitive",
		      have_browser && not_small, NULL);
	if (have_browser) {
		gboolean shown = eel_gconf_get_boolean (header->priv->browser_key);
		gtk_expander_set_expanded    (GTK_EXPANDER (header->priv->disclosure),
					      shown);
		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (viewbrowser_action),
					      shown);
	}
}

static void
rb_source_header_search_activate_cb (RBSearchEntry *search,
				     RBSourceHeader *header)
{
	RBEntryView *entry_view;

	entry_view = rb_source_get_entry_view (header->priv->selected_source);
	gtk_widget_grab_focus (GTK_WIDGET (entry_view));
}
